part of masamune.flutter;

/// Model for creating a collection.
@immutable
abstract class CollectionModel<T extends IDataCollection> {
  /// Path for models.
  final String path;

  /// Model for creating a collection.
  ///
  /// [path]: Path for models.
  @mustCallSuper
  CollectionModel(String path) : this.path = path?.applyTags() {
    if (PathMap.get<T>(this.path) != null) return;
    this.build(BuildModelContext._(this.path));
  }

  /// Perform the build.
  ///
  /// [context]: Build model context.
  FutureOr<T> build(BuildModelContext context);

  /// Reads the value while monitoring it.
  ///
  /// [context]: Build context.
  T watch(BuildContext context) {
    return context.watch<T>(this.path);
  }

  /// Reads the value.
  ///
  /// [context]: Build context.
  T read(BuildContext context) {
    return context.read<T>(this.path);
  }

  /// True. if the value exists.
  ///
  /// [context]: Build context.
  bool exist(BuildContext context) {
    return PathMap.get<T>(this.path) != null;
  }

  /// Get the number of elements of the value.
  ///
  /// [context]: Build context.
  int length(BuildContext context) {
    T tmp = PathMap.get<T>(this.path);
    if (tmp == null) return 0;
    return tmp.length;
  }
}
