part of masamune.flutter;

/// Model for creating a unit.
@immutable
abstract class UnitModel<T extends IDataField> {
  /// Path for models.
  final String path;

  /// Model for creating a unit.
  ///
  /// [path]: Path for models.
  @mustCallSuper
  UnitModel(String path) : this.path = path?.applyTags() {
    if (PathMap.get<T>(this.path) != null) return;
    this.build(BuildModelContext._(this.path));
  }

  /// Perform the build.
  ///
  /// [context]: Build model context.
  FutureOr<T> build(BuildModelContext context);

  /// Reads the value while monitoring it.
  ///
  /// [context]: Build context.
  TValue watch<TValue extends Object>(BuildContext context) {
    return context.watch<TValue>(this.path);
  }

  /// Reads the value.
  ///
  /// [context]: Build context.
  TValue read<TValue extends Object>(BuildContext context) {
    return context.read<TValue>(this.path);
  }

  /// True. if the value exists.
  ///
  /// [context]: Build context.
  bool exist(BuildContext context) {
    return PathMap.get<T>(this.path) != null;
  }
}
